'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var minimist = _interopDefault(require('minimist'));
var core = require('@dprint/core');
var fs = require('fs');
var path = require('path');
var globby = _interopDefault(require('globby'));

function parseCommandLineArgs(args) {
    const argv = minimist(args, {
        string: ["config"],
        boolean: ["help", "version", "outputFilePaths", "outputResolvedConfig", "allowNodeModuleFiles", "duration", "init"]
    });
    return {
        allowNodeModuleFiles: argv["allowNodeModuleFiles"],
        config: getConfigFilePath(),
        init: argv["init"],
        showHelp: argv["h"] || argv["help"],
        showVersion: argv["v"] || argv["version"],
        outputFilePaths: argv["outputFilePaths"],
        outputResolvedConfig: argv["outputResolvedConfig"],
        duration: argv["duration"],
        filePatterns: argv._
    };
    function getConfigFilePath() {
        return argv["c"] || argv["config"] || undefined;
    }
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function getPackageVersion() {
    return "0.7.0";
}

function getHelpText(plugins) {
    return `dprint v${getPackageVersion()}

Syntax:   dprint [options] [...file patterns]
Examples: dprint
          dprint "src/**/*.ts"
Options:
-h, --help              Outputs this message.
-v, --version           Outputs the version of the library and plugins.
--init                  Creates a dprint.config.js file in the current directory.
-c, --config            Configuration file to use (default: dprint.config.js)
--outputFilePaths       Outputs the list of file paths found for formatting without formatting the files.
--outputResolvedConfig  Outputs the resolved configuration from the configuration file.
--duration              Outputs how long the format took.
--allowNodeModuleFiles  Allows including files that have a node_modules directory in their path.
${getPluginTexts()}`;
    function getPluginTexts() {
        const prefix = "Plugins:";
        let result = prefix;
        if (plugins.length === 0)
            result += " [No plugins]";
        else {
            for (const plugin of plugins)
                result += `\n* ${plugin.name} v${plugin.version}`;
        }
        return result;
    }
}

function getVersionText(plugins) {
    return `dprint v${getPackageVersion()}${getPluginTexts()}`;
    function getPluginTexts() {
        let result = "";
        if (plugins.length === 0)
            result += " (No plugins)";
        else {
            for (const plugin of plugins)
                result += `\n${plugin.name} v${plugin.version}`;
        }
        return result;
    }
}

function throwError(message) {
    throw getError(message);
}
function getError(message) {
    return new Error(`[dprint]: ${message}`);
}

function readFile(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            fs.readFile(filePath, { encoding: "utf8" }, (err, text) => {
                if (err)
                    reject(err);
                else
                    resolve(text);
            });
        });
    });
}
function writeFile(filePath, text) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            fs.writeFile(filePath, text, { encoding: "utf8" }, err => {
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    });
}
function exists(fileOrDirPath) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            try {
                fs.exists(fileOrDirPath, result => {
                    resolve(result);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    });
}

function resolveConfigFile(filePath, environment) {
    return __awaiter(this, void 0, void 0, function* () {
        const resolvedFilePath = resolveConfigFilePath(filePath, environment);
        return {
            filePath: resolvedFilePath,
            config: yield getConfig()
        };
        function getConfig() {
            return __awaiter(this, void 0, void 0, function* () {
                let config;
                try {
                    config = yield environment.require(resolvedFilePath);
                }
                catch (err) {
                    if (yield environment.exists(resolvedFilePath))
                        return throwError(`Error loading configuration file '${resolvedFilePath}'.\n\n${err}`);
                    else if (filePath == null) {
                        return throwError(`Could not find configuration file at '${resolvedFilePath}'. `
                            + `Did you mean to create one (dprint --init) or specify a --config option?\n\n`
                            + err);
                    }
                    else {
                        return throwError(`Could not find specified configuration file at '${resolvedFilePath}'. Did you mean to create it?\n\n` + err);
                    }
                }
                if (typeof config !== "object" || typeof config.config !== "object")
                    return throwError(`Expected an object to be exported on the 'config' named export of the configuration at '${resolvedFilePath}'.`);
                else
                    return config.config;
            });
        }
    });
}
function resolveConfigFilePath(filePath, environment) {
    return environment.resolvePath(filePath || "dprint.config.js");
}

const projectTypeInfo = {
    values: [{
            name: "openSource",
            description: "Dprint is formatting an open source project."
        }, {
            name: "commercialSponsored",
            description: "Dprint is formatting a closed source commercial project and your company sponsored dprint."
        }, {
            name: "commercialDidNotSponsor",
            description: "Dprint is formatting a closed source commercial project and you want to forever enshrine your name "
                + "in source control for having specified this."
        }]
};
function getMissingProjectTypeDiagnostic(config) {
    const validProjectTypes = projectTypeInfo.values.map(v => v.name);
    if (validProjectTypes.includes(config.projectType || ""))
        return undefined;
    const propertyName = "projectType";
    const largestValueName = validProjectTypes.map(s => s.length).sort().pop();
    return {
        propertyName,
        message: `The "${propertyName}" field is missing. You may specify any of the following possible values in the configuration file according to your `
            + `conscience and that will supress this warning.\n\n`
            + projectTypeInfo.values.map(value => ` * ${value.name} ${" ".repeat(largestValueName - value.name.length)}${value.description}`).join("\n")
    };
}

function runCli(args, environment) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = parseCommandLineArgs(args);
        yield runCliWithOptions(options, environment);
    });
}
function runCliWithOptions(options, environment) {
    return __awaiter(this, void 0, void 0, function* () {
        const startDate = new Date();
        if (options.showHelp) {
            environment.log(getHelpText(yield safeGetPlugins()));
            return;
        }
        else if (options.showVersion) {
            environment.log(getVersionText(yield safeGetPlugins()));
            return;
        }
        else if (options.init) {
            yield createConfigFile(environment);
            return;
        }
        const { unresolvedConfiguration, configFilePath, plugins } = yield getUnresolvedConfigAndPlugins();
        const globalConfig = resolveGlobalConfigurationInternal();
        updatePluginsWithConfiguration();
        const filePaths = yield getFilePaths();
        if (options.outputFilePaths) {
            for (const filePath of filePaths)
                environment.log(filePath);
            return;
        }
        else if (options.outputResolvedConfig) {
            outputResolvedConfiguration();
            return;
        }
        const promises = [];
        for (const filePath of filePaths) {
            const promise = environment.readFile(filePath).then(fileText => {
                const result = core.formatFileText({
                    filePath,
                    fileText,
                    plugins
                });
                return result === fileText ? Promise.resolve() : environment.writeFile(filePath, result);
            }).catch(err => {
                const errorText = err.toString().replace("[dprint]: ", "");
                environment.error(`Error formatting file: ${filePath}\n\n${errorText}`);
            });
            promises.push(promise);
        }
        return Promise.all(promises).then(() => {
            if (options.duration) {
                const durationInSeconds = ((new Date()).getTime() - startDate.getTime()) / 1000;
                environment.log(`Duration: ${durationInSeconds.toFixed(2)}s`);
            }
        });
        function resolveGlobalConfigurationInternal() {
            const missingProjectTypeDiagnostic = getMissingProjectTypeDiagnostic(unresolvedConfiguration);
            const configResult = core.resolveConfiguration(getUnresolvedConfigStrippedOfCliSpecificConfig());
            for (const diagnostic of configResult.diagnostics)
                warnForConfigurationDiagnostic(diagnostic);
            if (missingProjectTypeDiagnostic)
                warnForConfigurationDiagnostic(missingProjectTypeDiagnostic);
            return configResult.config;
            function getUnresolvedConfigStrippedOfCliSpecificConfig() {
                const obj = Object.assign({}, unresolvedConfiguration);
                delete obj.excludes;
                delete obj.includes;
                return obj;
            }
        }
        function updatePluginsWithConfiguration() {
            for (const plugin of plugins) {
                plugin.initialize({
                    environment,
                    globalConfig
                });
                for (const diagnostic of plugin.getConfigurationDiagnostics())
                    warnForConfigurationDiagnostic(diagnostic);
            }
        }
        function getFilePaths() {
            return __awaiter(this, void 0, void 0, function* () {
                const isInNodeModules = /[\/|\\]node_modules[\/|\\]/i;
                const allFilePaths = yield environment.glob(getFileGlobs());
                return options.allowNodeModuleFiles
                    ? allFilePaths
                    : allFilePaths.filter(filePath => !isInNodeModules.test(filePath));
                function getFileGlobs() {
                    return [...getIncludes(), ...getExcludes()];
                    function getIncludes() {
                        if (options.filePatterns.length > 0) {
                            if (!options.outputFilePaths && unresolvedConfiguration.includes && unresolvedConfiguration.includes.length > 0)
                                environment.warn("Ignoring the configuration file's includes because file patterns were provided to the command line.");
                            return options.filePatterns;
                        }
                        return unresolvedConfiguration.includes || [];
                    }
                    function getExcludes() {
                        if (!unresolvedConfiguration.excludes)
                            return [];
                        return unresolvedConfiguration.excludes.map(pattern => {
                            if (pattern.startsWith("!"))
                                return pattern;
                            return "!" + pattern;
                        });
                    }
                }
            });
        }
        function warnForConfigurationDiagnostic(diagnostic) {
            environment.warn(`[${environment.basename(configFilePath)}]: ${diagnostic.message}`);
        }
        function outputResolvedConfiguration() {
            environment.log(getText());
            function getText() {
                let text = `Global configuration: ${prettyPrintAsJson(globalConfig)}`;
                for (const plugin of plugins)
                    text += `\n${plugin.name}: ${prettyPrintAsJson(plugin.getConfiguration())}`;
                return text;
            }
            function prettyPrintAsJson(obj) {
                const numSpaces = 2;
                return JSON.stringify(obj, null, numSpaces);
            }
        }
        function safeGetPlugins() {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    return (yield getUnresolvedConfigAndPlugins()).plugins;
                }
                catch (err) {
                    return [];
                }
            });
        }
        function getUnresolvedConfigAndPlugins() {
            return __awaiter(this, void 0, void 0, function* () {
                const { config: unresolvedConfiguration, filePath: configFilePath } = yield resolveConfigFile(options.config, environment);
                return {
                    unresolvedConfiguration,
                    configFilePath,
                    plugins: unresolvedConfiguration.plugins || []
                };
            });
        }
    });
}
function createConfigFile(environment) {
    return __awaiter(this, void 0, void 0, function* () {
        const filePath = resolveConfigFilePath(undefined, environment);
        if (yield environment.exists(filePath)) {
            environment.warn(`Skipping initialization because a configuration file already exists at: ${filePath}`);
            return;
        }
        environment.writeFile(filePath, getDefaultConfigFileText());
        environment.log(`Created ${filePath}`);
        function getDefaultConfigFileText() {
            return `// @ts-check
const { TypeScriptPlugin } = require("dprint-plugin-typescript");
const { JsoncPlugin } = require("dprint-plugin-jsonc");

/** @type { import("dprint").Configuration } */
module.exports.config = {
    projectType: "openSource",
    plugins: [
        new TypeScriptPlugin({
        }),
        new JsoncPlugin({
            indentWidth: 2
        })
    ],
    includes: [
        "**/*{.ts,.tsx,.json,.js}"
    ]
};
`;
        }
    });
}

class CliEnvironment extends core.CliLoggingEnvironment {
    basename(fileOrDirPath) {
        return path.basename(fileOrDirPath);
    }
    resolvePath(fileOrDirPath) {
        return path.normalize(path.resolve(fileOrDirPath));
    }
    readFile(filePath) {
        return readFile(filePath);
    }
    writeFile(filePath, text) {
        return writeFile(filePath, text);
    }
    exists(filePath) {
        return exists(filePath);
    }
    glob(patterns) {
        return globby(patterns, { absolute: true });
    }
    require(filePath) {
        return new Promise((resolve, reject) => {
            try {
                resolve(require(filePath));
            }
            catch (err) {
                reject(err);
            }
        });
    }
}

exports.CliEnvironment = CliEnvironment;
exports.runCli = runCli;
